1 ICON "mednaffe.ico"

1 VERSIONINFO
FILEVERSION     0,9,2,0
PRODUCTVERSION  0,9,2,0
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904E4"
    BEGIN
      VALUE "FileDescription", "A front-end (GUI) for Mednafen emulator"
      VALUE "FileVersion", "0.9.2"
      VALUE "InternalName", "mednaffe"
      VALUE "LegalCopyright", "Copyright \251 2010-2021 AmatCoder"
      VALUE "LegalTrademarks", "Released under GPLv3"
      VALUE "OriginalFilename", "mednaffe.exe"
      VALUE "ProductName", "Mednaffe"
      VALUE "ProductVersion", "0.9.2"
    END
  END

  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x409, 1252
  END
END
